/*  DFP optimization routine procedures    (M Boldin 11/95)          */
/*  based on GAUSS routines that were originally written by          */
/*    Bo Honore and Ekaterini Kyriazidou                             */
/*  modifications                                                    */
/*      5/96-- fixed divsion by zero problem in monitx(.)            */

external matrix ftol, maxsec, maxit, fret_l, tim_l;
external matrix g;
external string xhead;
external matrix __oiter, __otime;

declare ftol=10E-6;
declare maxsec=3600;
declare maxit=200;
declare fret_l=0;
declare tim_l=0;
declare string xhead="DFP routine";
declare g;
declare __oiter=0;
declare __otime=0;


end;

/* Monitor output from iteration */
proc (0)=monitx(p,fret,tim,iter);
   local chf, rchf;
     chf=(fret-fret_l);  rchf=0;
     if abs(chf) gt 10E-15;  rchf=100*ftol/abs(chf);  endif;
cls;
print;  format /rd  6,2;  outwidth 73;
locate 1,1;
"=====================================================================";
$xhead;
"Iteration:  "  iter
" Iteration seconds: "  tim-tim_l
" Total-- "  tim   tim/iter;
format /rd  8,4;
"-lnLik value: "  fret
" change "  chf  rchf;
"=====================================================================";
" Parameters: "  p';
" RGradients: "  (g .* p / fret)';
print;
fret_l=fret;  tim_l=tim;

endp;




proc (1) = gradx(ff,xx);
 local gg;
 gg=gradp(ff,xx);
 g=gg';
 retp(g);
endp;


/* The following code is from the GAUSS archive at American University */
/*   under "Numerical Recipes" by Honore and Kyriazidou                */
/*   Not all are included-- DFPMINX is an ammended version of DFPMIN   */
/*   It allows the use of more general gradient procs & alters the     */
/*   iteration output using MONITX                                     */
/*   The basic minimization (-optimization) algorithm is unchanged     */


/************* OPTIMIZATION ROUTINES  ******************************
The following are GAUSS versions of the optimization routines in
"Numerical Recipes" written by Bo Honore and Ekaterini Kyriazidou
of Northwestern University, with support from an NSF grant.
You are welcome to use and distribute them as long as you

include proper attribution to the authors.
You should know that you use the routines at your own risk.
THE ROUTINES FOLLOW:

***********************************************************/



PROC (2) = dfpminx(p,ftol,maxsec,maxit,&fct,&dfct,&linm);

/* Changed to allow use of GAUSS's GRADP for &dftc           */
/*    see the proc gradx(&ff,p) above                        */
/*    now  dfct requires two arguments   -- M Boldin 11/95   */

/* This procedure minimizes a function called FCT using the
   Broyden-Fletcher-Goldfarb-Shanno (BFGS) method.
   The procedure is tailored after DFPMIN in Numerical Recipes.

   INPUT

   p      (nx1)    starting vector
   ftol   (1x1)    fractional convergence tolerance in the function value
   maxsec (1x1)    maximum number of seconds allowed for running program
   maxit  (1x1)    maximum number of iterations allowed
   fct    proc     function we want to minimize
   dfct   proc     routine calculating  the gradient of the function
   linm   proc     routine performing line minimization

   OUTPUT

   p      (nx1)    minimizing vector
   hessin

*/

    local tim,date1,fp,xi,g,hessin,hdg,dg,iter,eps,fret,n,tol,fac,fad,fae;
    local  fct:proc,dfct:proc,linm:proc;

    tim=0;
    date1=date;
    __otime=date1;
    eps=0.0000000001;
    tol=0.00001;
    n=rows(p);
    fp=fct(p);
    g=dfct(&fct,p);  /* was-- g=dfct(p); */
    xi=-g;
    hessin=eye(n);
    iter=1;
    do while iter<=maxit;
    __oiter=iter;
    {p,fret}=linm(p,xi,tol,&fct);
    tim=ethsec(date1,date)/100;
    call monitx(p,fret,tim,iter);
    if ((2*abs(fp-fret)).le ftol*(abs(fp)+abs(fret)+eps));
    RETP(p,hessin);
    endif;
    if (tim .ge maxsec); "Maximum number of seconds exceeded";
    RETP(p,hessin);
    endif;
    fp=fct(p);
    dg=g;
    fret=fct(p);
    g=dfct(&fct,p);
    dg=g-dg;
    hdg=hessin*dg;
    fac=dg'*xi;
    fae=dg'*hdg;
    if (fac==0); RETP(p,hessin); else; fac=1/fac; endif;
    if (fae==0); RETP(p,hessin); else; fad=1/fae; endif;
    dg=fac*xi-fad*hdg;
    hessin=hessin+fac*xi*xi'-fad*hdg*hdg'+fae*dg*dg';
    xi=-hessin*g;
    iter=iter+1;
    endo; "Maximum number of iterations exceeded";
    RETP(p,hessin);
ENDP;

PROC (2) = linmib(pst,xi,tol,&fct);

/* This routine performs line minimization of a (n-dimensional) function
   called FCT, using Brent's one-dimensional minimization method without
   derivatives. It is tailored after BRENT in Numerical Recipes, starting
   off with an ad hoc initial bracketing of the minimum. Given as inputs
   a vector P and a direction vector XI, it minimizes FCT(P+X*XI) w/r/t
   the scalar X, isolating X (the best abscissa) at the midpoint of two
   scalars, a & b, that are 2*X*TOL apart, i.e. at a fractional precision
   of about TOL. The maximum allowed number of iterations is 100.

   INPUT

   pst    (nx1)     starting value
   xi     (nx1)     direction vector
   tol    (1x1)     fractional tolerance
   fct    proc      function we want to minimize

   OUTPUT

   pst+x*xi    (nx1)     minimizing vector along direction xi; new vector
                         direction
   fx          (1x1)     value of fct at minimum
*/

    local maxit,cgold,zeps,a1,a2,ad,am,f1,f2,fm,itry,
          v,u,w,fu,fv,fw,x,fx,tol1,tol2,e,etemp,iter,q,r,a,b,xm,d,p,fct:proc;

    cgold=0.3819660;  maxit=100; zeps=.0000000001;
    a1=0;
    a2=0.01;
    f1=fct(pst+a1*xi);
    itry=0;
  be:
    itry=itry+1;
    if (itry.eq 10); "linmib itry=10"; stop; endif;
    f2=fct(pst+a2*xi);
    if(f1.eq f2); a2=a2+1; goto be; endif;
    if (f1.le f2);
    am=a1; fm=f1;
    f1=f2; a1=a2;
    else;
    am=a2; fm=f2;
    endif;
    ad=5*(am-a1);
    a2=am+ad;

    f2=fct(pst+a2*xi);
    do until (f2.gt fm);
    ad=2*ad;
    a2=am+ad;
    f2=fct(pst+a2*xi);
    endo;
    if (a1.<a2);
    a=a1; b=a2;
    else;
    a=a2; b=a1;
    endif;
    v=am; x=v; w=v;
    e=0;

    fx=fct(pst+x*xi); fv=fx; fw=fx;


    iter=1;
    do while (iter <=maxit);
    xm=(a+b)*0.5;
    tol1=tol*abs(x)+zeps;
    tol2=2*tol1;
    if (abs(x-xm).le (tol2-0.5*(b-a))); goto lb3; endif;
    if (abs(e).gt tol1);
        r=(x-w)*(fx-fv);
        q=(x-v)*(fx-fw);
        p=(x-v)*q-(x-w)*r;
        q=2*(q-r);
        if (q.gt 0); p=-p; endif;
        q=abs(q);
        etemp=e;
        e=d;
        if ((abs(p).ge abs(0.5*q*etemp)).or (p.le q*(a-x))
        .or (p.ge (q*(b-x)))); goto lb1; endif;
        d=p/q;
        u=x+d;
        if (((u-a).lt tol2).or ((b-u).lt tol2));
        d=abs(tol1)*((xm.>x)-(xm.<x));
        endif;
        goto lb2;
    endif;
      lb1:
    if (x.ge xm);
        e=a-x;
    else;
        e=b-x;
    endif;
    d=cgold*e;
      lb2:
    if (abs(d).ge tol1);
        u=x+d;
    else;
        u=x+abs(tol1)*((d.>0)-(d.<0));
    endif;
    fu=fct(pst+u*xi);
    if (fu.le fx);
        if (u.ge x); a=x; else; b=x; endif;
        v=w; fv=fw; w=x; fw=fx; x=u; fx=fu;
    else;
        if (u.lt x); a=u; else; b=u; endif;
        if ((fu.le fw).or (w.eq x));
        v=w; fv=fw; w=u; fw=fu;
        elseif ((fu.le fv).or (v.eq x).or (v.eq w));
        v=u; fv=fu;
        endif;
    endif;
    iter=iter+1;
    endo;
    "BRENT EXCEEDED MAXIMUM NUMBER OF ITERATIONS";
  lb3:

RETP(pst+x*xi,fx);
ENDP;
PROC (2) = linmin(p,xi,tol,&fct);

/*
   This routine performs an ad hoc n-dimensional Golden Section Search for
   the minimum of a function called FCT.

   INPUT

   pi    (nx1)    vector of starting values
   xi    (nx1)    direction vector
   tol   (1x1)    tolerance
   fct   proc     function we want to minimize

   OUTPUT

   p     (nx1)    minimizing vector
   f     (nx1)    value of fct at minimum,p
*/

    local f1,f2,fm,f,p1,p2,pm,xd,x1,x2,d1,d2,fx1,fx2,itry,fct:proc;
    /*  local f1,f2,fm,f,p1,p2,pm,p,xd,x1,x2,d1,d2,fx1,fx2,itry,fct:proc; */
    d1=0.618; d2=0.382;
    p1=p;
    p2=p+xi;
    f1=fct(p1);
    itry=0;
  be:
    itry=itry+1;
    if (itry.eq 10); "linmin itry=10"; stop; endif;
    f2=fct(p2);
    if(f1.eq f2); p2=p2+xi; goto be; endif;
    if (f1.le f2);
    pm=p1; fm=f1;
    f1=f2; p1=p2;
    else;
    pm=p2; fm=f2;
    endif;
    xd=pm-p1;
    p2=pm+xd;
    f2=fct(p2);
    do until (f2.gt fm);
    xd=2*xd;
    p2=pm+xd;
    f2=fct(p2);
    endo;
/* p1'; pm'; p2'; */
    stit:
    x1=d1*p1+d2*p2;
    x2=d2*p1+d1*p2;
    fx1=fct(x1);
    fx2=fct(x2);
    if (fx1.le fx2);
    p2=x2;
    else;
    p1=x1;
    endif;
    if (sqrt(sumc((p1-p2).^2)).ge tol); goto stit; endif;
    if (fx1.lt fx2);
    p=x1;
    else;
    p=x2;
    endif;
    f=fct(p);
RETP(p,f);
ENDP;

