/* MSMGNP  M Boldin  Markov Switching model for GNP growth */;

disable;
output file=mgnp2o.out off;

clear ddates, y, tn, X, depvar, indvar, dvar, ivar, x0i;

load path=c:;
gosub GETDATA;
/*  SETMODEL subroutine loads data and defines y and X matrices
    Variable columns and names:
      QDATA variables
         1-YEARQ  2-pchGNP  3-CLTR
         4-CLTRl1 5-chCLTR  6-FUELP
      X0 variables
         1-CONST  2-TTREND;
*/

__ftt=5;    /* 1952:2 */   __ltt=155;  /* 1989:4 */
Mtitle="GNP growth Model" ; _otitle=Mtitle;
dvar=2; let x0i=1; let ivar=0;  /* just constant used */
gosub SETMODEL;

call msminit;

/* 1st run, Hamilton's MSM specification (Econometrica, 1989)  */
ns=2;  __nxsees=1;   arn=4;   __rhosx=1.1;
let sval0 =  -.25    1.25
              1
             0 0 0 0
             .75
             .25;
call msmr(sval0);


/* 2nd run for more general AR(4) MSM  */
/*   using previous model as starting values  */
ns=2;  __nxsees=0;   arn=4;   __rhosx=0;
sval0=vec(bs')|vec(sees)|vec(rhos')|xq;
call msmr(sval0);

/* 3rd run same AR(4) MSM, different starting values  */
let sval0 =
    -0.3029    1.0760
     0.9614    0.7511
     0  0
     0  0
     0  0
     0  0
     0.7305
     0.0902;
call msmr(sval0);

end;



GETDATA:

/* Loading Data  */
  load xdat[155,6]=qdata.asc;
  let vnms=  YEARQ  pchGNP  CLTR  CLTRL1   chCLTR  FUELP;
  ddates=xdat[.,1];
  tn=rows(xdat);
  X0=ones(tn,1)~seqa(1,1,tn)/100;
  let x0vn= CONST  TTREND;

return;


SETMODEL:
clear y, tn, X, depvar, indvar;

 /* y - dependent (LHS) variable   */
   y=xdat[.,dvar];  depvar=vnms[dvar];
   tn=rows(y);

 /*  X - independent (RHS) variables  */
  if x0i ne 0;  X=X0[.,x0i];  indvar=x0vn[x0i];  endif;
  if ivar[1]==0 and x0i==0;  X=X0[.,1];  indvar=x0vn[1];  endif;
  if ivar[1] ne 0  and x0i ne 0;
     X=X~xdat[.,ivar];  indvar=indvar|vnms[ivar];
  endif;
  if ivar[1] ne 0  and x0i == 0;
     X=xdat[.,ivar];  indvar=vnms[ivar];
  endif;

  output on;
  print;
  $Mtitle;
  "Depvar -- "  $depvar;
  "  mean/std" meanc(y[__ftt:__ltt])   stdc(y[__ftt:__ltt]);
  "Indvar --"  $indvar';
   print;
  output off;

return;


