/* MSMGNP  M Boldin  Markov Switching model for GNP growth */;

disable;
output file=mgnp3o.out off;

clear ddates, y, tn, X, depvar, indvar, dvar, ivar, x0i;

load path=c:;
gosub GETDATA;
/*  SETMODEL subroutine loads data and defines y and X matrices
    Variable columns and names:
      QDATA variables
         1-YEARQ  2-pchGNP  3-CLTR
         4-CLTRl1 5-chCLTR  6-FUELP
      X0 variables
         1-CONST  2-TTREND;
*/

__ftt=5;    /* 1952:2 */   __ltt=155;  /* 1989:4 */
Mtitle="GNP growth Model" ; _otitle=Mtitle;
dvar=2; let ivar= 0;  /* with=0 just constant used */
gosub SETMODEL;

call msminit;
ns=3;  __nxsees=0;   arn=0;   __rq=2;
/*  Here, a transformation (__rq=2) is used to keep Q elements between 0-1 */

/*  Starting Values  */;
let sval0 =  -1   0   1.25
             .8   .5   .8;
let q0[3,2]= .50  .25
             .25  .50
             .25  .25;
sval0=sval0|vec(qtxq(q0)');
call msmr(sval0);


/*  3 state  AR(2) MSM */
/*    using previous AR(0) MSM  as starting values */
ns=3;  __nxsees=0;   arn=2;    __rq=2;
sval0=vec(bs')|sees'|zeros(arn*ns,1)|vec(qtxq(q[.,1:ns-1])');
call msmr(sval0);

/*  3 state  AR(2) MSM */
/*    with different starting values & Q restriction (__rq=1)
        and likelihood function proc                            */
ns=3;  __nxsees=0;   arn=2;   __rq=1;
let sval0 = -.5   .5   1.5
             .7   .7   .7
              0    0    0
              0    0    0;
let q0= .70  .05
        .15  .70
        .05  .40;
sval0=sval0|q0;
call msmr(sval0);


/*  3 regimes, restrictions on sees & diagonal AR(1)        */
/*    also restricted (diagonal) Q-- using restrq(.) proc   */
call msminit;
ns=3;  __nxsees=1;   arn=1;   __ar1m=2;  __nxrhos=1;
__rq=3;  __nxq=3;
let sval0 =
-0.25     0.0    1.25
 0.80
   0
 0.70
       0.50
            0.90;
call msmr(sval0);

ns=3;  __nxsees=0;   arn=1;   __ar1m=2;  __nxrhos=0;
sval0=vec(bs')|sees'|vec(xrhos[1 1 1])|xq;
call msmr(sval0);


end;

/* Proc for restricted (diagonal) Q matrix */
proc rxq(xqx);
  local qn, qq, jj;   qq=zeros(ns,ns);
  qq[1,1]=xqx[1];   qq[1,2]=1-qq[1,1];
  if ns gt 2;
  jj=1;  do until jj==ns-1; jj=jj+1;
     qq[jj,jj]=xqx[jj];  qq[jj,jj+1]=1-qq[jj,jj];
  endo;
  endif;
  qq[ns,ns]=xqx[ns];  qq[ns,1]=1-qq[ns,ns];
  qq=rq(qq);
  retp(qq);
endp;



GETDATA:

/* Loading Data  */
  load xdat[155,6]=qdata.asc;
  let vnms=  YEARQ  pchGNP  CLTR  CLTRL1   chCLTR  FUELP;
  ddates=xdat[.,1];
  tn=rows(xdat);
  X0=ones(tn,1)~seqa(1,1,tn)/100;
  let x0vn= CONST  TTREND;

return;


SETMODEL:
clear y, tn, X, depvar, indvar;

 /* y - dependent (LHS) variable   */
   y=xdat[.,dvar];  depvar=vnms[dvar];
   tn=rows(y);

 /*  X - independent (RHS) variables  */
  if x0i ne 0;  X=X0[.,x0i];  indvar=x0vn[x0i];  endif;
  if ivar[1]==0 and x0i==0;  X=X0[.,1];  indvar=x0vn[1];  endif;
  if ivar[1] ne 0  and x0i ne 0;
     X=X~xdat[.,ivar];  indvar=indvar|vnms[ivar];
  endif;
  if ivar[1] ne 0  and x0i == 0;
     X=xdat[.,ivar];  indvar=vnms[ivar];
  endif;

  output on;
  print;
  $Mtitle;
  "Depvar -- "  $depvar;
  "  mean/std" meanc(y[__ftt:__ltt])   stdc(y[__ftt:__ltt]);
  "Indvar --"  $indvar';
   print;
  output off;

return;


