/* MSMMD1    M Boldin
   Markov Switching model for Monthly data
*/;

output file=mmd1o.out off;

clear   y, tn, X, depvar, indvar, dvar, ivar, ivs, x0i;
clear   xdat, ddates, nbers, x0, x0vn;

load path=c:;
gosub GETDATA;
/*  First, GETDATA to initialize  complete data set
          Below SETMODEL subroutine loads data and define y and X matrices
       variable columns and names
   1 Year      5 LTE/LNN    9 CUMFG     13 IP
   2 Month     6 LRM20     10 LTE       14 IPMFG
   3 NBERR     7 LRW20     11 LANAGRA   15 IPDG
   4 LTR      8 LR16      12 LAPRIVA   16 IPND
*/;

__ftt=121;  __ltt=480;  /* 1960 -- 1989 */
Mtitle="Monthly BC Models" ; _otitle=Mtitle;

call msminit;
ns=2;   __nxsees=0;  arn=0;  __rq=1;
__bsk=0;


dvar=4;  lndiffc=0;  lagn=2;  let x0i = 1 2;
gosub SETMODEL;
    k1=cols(X);
    b1=y[__ftt:__ltt]/X[__ftt:__ltt,.];
    see1=stdc(y[__ftt:__ltt]-X[__ftt:__ltt,.]*b1);
    bs0=b1[1]+see1/2|b1[1]-see1/2|b1[2:k1];

    jj=seqa(1,1,2);
    nbr=nbers[__ftt:__ltt];  Xn=dummybr(nbr,jj);
    Xn=Xn~X[__ftt:__ltt,2:k1];
    bn=y[__ftt:__ltt]/Xn;
    seen=stdc(y[__ftt:__ltt]-Xn*bn);


modeld="2A"; let __bsk = 1 2 3 3 4 4 5 5;
__nxsees=1; __rq=3; __nxq=1;
sval0=bs0|see1|.95;
call msmr(sval0);

modeld="2B";  let __bsk=0;  __nxsees=0;  __rq=3;  __nxq=1;
sval0=vec(bs')|sees'|q[1,1];
call msmr(sval0);

modeld="2C";  let __bsk=0;  __nxsees=0;  __rq=1;  __nxq=0;
sval0=vec(bs')|sees'|q[.,1];
call msmr(sval0);

end;


proc rxq(xqq); local qq;  qq=zeros(2,1);
  qq[1,1]=xqq[1];  qq[2,1]=1-xqq[1];
  qq=rq(qq~(1-sumc(qq')));
  retp(qq);
endp;


proc (1) = lags(xd,al);
  local nxd, ncd, nals, j, alj, xdj, xdo;
  nxd=rows(xd);  ncd=cols(xd); nals=rows(al);
  j=1;  alj=al[j];
  xdo=( miss(zeros(alj,ncd),0) | xd[1:nxd-alj,.] );
  do until j==nals;  j=j+1;    alj=al[j];
     xdj = ( miss(zeros(alj,ncd),0) | xd[1:nxd-alj,.] );
     xdo=xdo~xdj;
  endo;
  retp(xdo);
endp;

proc (1) = lndiff(xd);  local lnd, xdl;
  xdl=lags(xd,1);  lnd=ln(xd)-ln(xdl);
 retp(lnd);
endp;



GETDATA:
/* Loading Data  */
   load vnms[16,1]=mdata.var;
   load xdat1[480,12]=mdata1.asc;
   load xdat2[480,6]=mdata2.asc;
   xdat=xdat1~xdat2[.,3:6];

   tn=rows(xdat);
   X0=ones(tn,1)~seqa(1,1,tn)/100;
   let x0vn= CONST  TTREND;

ddates = xdat[.,1]+xdat[.,2]/12-.5/12;  nbers=2-xdat[.,3];

return;


SETMODEL:
/* y - dependent (LHS) variable   */
   y=xdat[.,dvar];  depvar=vnms[dvar];
   if lndiffc==1;  y=100*lndiff(y);  endif;
   tn=rows(y);

/* X - independent (RHS) variables  */
   if x0i ne 0;  X=X0[.,x0i];  indvar=x0vn[x0i];  endif;
   if ivar[1]==0 and x0i==0;  X=X0[.,1];  indvar=x0vn[1];  endif;
   if ivar[1] ne 0  and x0i ne 0;
      X=X~xdat[.,ivar];  indvar=indvar|vnms[ivar];
   endif;
   if ivar[1] ne 0  and x0i == 0;
      X=xdat[.,ivar];  indvar=vnms[ivar];
   endif;
   if lagn ge 1; lj=seqa(1,1,lagn);
      vly = ones(lagn,1)*("YL(") $+ ftocv(lj,2,0) $+ ones(lagn,1)*(")");
      if (X==0); X=lags(y,lj); indvar=vly;
         else; X=X~lags(y,lj);  indvar=indvar|vly; endif;
   endif;

  output on;
  print;
  $Mtitle;
  "Depvar -- "  $depvar;
  "  mean/std" meanc(y[__ftt:__ltt])   stdc(y[__ftt:__ltt]);
  "Indvar --"  $indvar';
   print;
  output off;

return;


